<?php

namespace App\Service;

use App\Entity\Gallery;
use App\Entity\Kitchen;
use App\Entity\Restaurant;
use App\Entity\RestaurantType;
use App\Exception\RestaurantNotFoundException;
use App\Repository\Interface\GalleryRepositoryInterface;
use App\Requests\RestaurantListRequest;
use App\Mapper\RestaurantMapper;
use App\Model\RestaurantDetailElement;
use App\Model\RestaurantList;
use App\Repository\Interface\KitchenRepositoryInterface;
use App\Repository\Interface\RestaurantRepositoryInterface;
use App\Repository\Interface\RestaurantTypeRepositoryInterface;
use Ramsey\Collection\Collection;
use Symfony\Component\HttpFoundation\Request;

class RestaurantService
{
    private const DEFAULT_PAGE  = 1;
    private const DEFAULT_LIMIT = 12;

    public function __construct(
        private RestaurantRepositoryInterface $restaurantRepository,
        private RestaurantTypeRepositoryInterface $restaurantTypeRepository,
        private KitchenRepositoryInterface $kitchenRepository,
        private GalleryRepositoryInterface $galleryRepository,
    ) {}

    public function getRestaurantsByRequest(
        RestaurantListRequest $request
    ): RestaurantList {
        $page = $request->getRequest()->query->get('page') ?? self::DEFAULT_PAGE;
        $limit = $request->getRequest()->query->get('limit') ?? self::DEFAULT_LIMIT;
        $restaurantTypeId = $request->getRequest()->query->get('restaurant_type_id');
        $kitchenId = $request->getRequest()->query->get('kitchen_id');

        return $this->getRestaurants(
            $page, $limit, $restaurantTypeId, $kitchenId
        );
    }

    public function getRestaurants(
        int $page,
        int $limit,
        string|null $restaurantTypeId,
        string|null $kitchenId
    ): RestaurantList {
        $restaurants = new Collection(
            Restaurant::class,
            $this->restaurantRepository->getAll(
                $page,
                $limit,
                $restaurantTypeId,
                $kitchenId
            )
        );
        $count = $this->restaurantRepository->getCount();
        $restaurantTypes = new Collection(
            RestaurantType::class,
            $this->restaurantTypeRepository->getAll()
        );
        $kitchens = new Collection(
            Kitchen::class,
            $this->kitchenRepository->getAll()
        );

        return RestaurantMapper::mapToRestaurantList(
            $restaurants,
            $restaurantTypes,
            $kitchens,
            $page,
            $limit,
            $count
        );
    }

    public function getRestaurantByRequest(Request $request): RestaurantDetailElement
    {
        return $this->getRestaurant(
            $request->get('restaurantId')
        );
    }

    public function getRestaurant(string $id): RestaurantDetailElement
    {
        $restaurant = $this->restaurantRepository->getById($id);
        if ($restaurant === null) {
            throw new RestaurantNotFoundException();
        }

        $gallery = new Collection(
            Gallery::class,
            $this->galleryRepository->getGalleryByRestaurantId($restaurant->getId())
        );

        return RestaurantMapper::mapToDetailElement($restaurant, $gallery);
    }
}